@@ -385,27 +385,25 @@ class JsonResource(Resource):
         except AssetNotFoundError:
             return False
 
-    def delete_jsonpath(self, json_path:str, ensure_exists:bool = False) -> None:
+    def delete_jsonpath(self, json_path:str) -> None:
         """
         Removes value at jsonpath location.
         """
         path_exists = self.jsonpath_exists(json_path)
         if path_exists:
             dpath.util.delete(self.data, json_path)
-        elif ensure_exists:
-            raise AssetNotFoundError(f"Path {json_path} does not exist. Cannot delete.")
 
-    def pop_jsonpath(self, json_path, default=NO_ARGUMENT, ensure_exists=False) \
+    def pop_jsonpath(self, json_path, default=NO_ARGUMENT) \
         -> Union[dict, list, int, str, float]:
         """
         Removes value at jsonpath location, and returns it.
         """
 
         data = self.get_jsonpath(json_path, default=default)
-        self.delete_jsonpath(json_path, ensure_exists=ensure_exists)
+        self.delete_jsonpath(json_path)
         return data
 
-    def set_jsonpath(self, json_path:str, insert_value:any, must_exist:bool=False, overwrite:bool=True):
+    def set_jsonpath(self, json_path:str, insert_value:any, overwrite:bool=True):
         """
         Sets value at jsonpath location.
 
@@ -414,11 +412,6 @@ class JsonResource(Resource):
 
         path_exists = self.jsonpath_exists(json_path)
 
-        # If the path must exist, and is missing, we can
-        # raise an error by getting the path
-        if not path_exists and must_exist:
-            raise AssetNotFoundError(f"Path {json_path} does not exist. Cannot set value.")
-
         # If overwrite is false, it will set the path only
         # if there is no path.
         if path_exists and not overwrite:
@@ -696,8 +689,8 @@ class TextureDouble(JsonSubResource):
         return self.data
     
     @texture_path.setter
-    def identifier(self, identifier):
-        self.data = identifier
+    def texture_path(self, texture_path):
+        self.data = texture_path
     
     def exists(self) -> bool:
         """
@@ -705,6 +698,40 @@ class TextureDouble(JsonSubResource):
         """
         return os.path.exists(os.path.join(self.pack.input_path, self.file_path))
 
+class MaterialTriple(JsonSubResource):
+    """
+    A special sub-resource, which represents a material within an RP entity.
+    """
+    def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
+        super().__init__(data=data, parent=parent, json_path=json_path)
+
+    @property
+    def shortname(self):
+        return self.id
+
+    @shortname.setter
+    def shortname(self, shortname):
+        self.id = shortname
+    
+    @property
+    def identifier(self):
+        return self.data
+    
+    @identifier.setter
+    def identifier(self, identifier):
+        self.data = identifier
+
+    @cached_property
+    def resource(self):
+        return self.parent.pack.get_material(self.identifier)
+    
+    def exists(self):
+        try:
+            self.parent.pack.get_material(self.identifier)
+            return True
+        except AssetNotFoundError:
+            return False
+
 @dataclass
 class Translation:
     """
@@ -1041,16 +1068,19 @@ class Project():
 class ResourcePack(Pack):
     def __init__(self, input_path: str, project: Project = None):
         super().__init__(input_path, project=project)
-        self.__particles: ParticleFile = []
-        self.__attachables: AttachableFileRP = []
-        self.__animation_controller_files: AnimationControllerFileRP = []
-        self.__animation_files: AnimationControllerFileRP = []
-        self.__entities: EntityFileRP = []
-        self.__model_files: ModelFile = []
-        self.__render_controller_files: RenderControllerFile = []
-        self.__items: ItemFileRP = []
+        self.__particles: list[ParticleFile] = []
+        self.__attachables: list[AttachableFileRP] = []
+        self.__animation_controller_files: list[AnimationControllerFileRP] = []
+        self.__animation_files: list[AnimationControllerFileRP] = []
+        self.__entities: list[EntityFileRP] = []
+        self.__model_files: list[ModelFile] = []
+        self.__models: list[Model] = []
+        self.__render_controller_files: list[RenderControllerFile] = []
+        self.__items: list[ItemFileRP] = []
         self.__sounds: list[str] = []
         self.__textures: list[str] = []
+        self.__material_files: list[MaterialFile] = []
+        self.__materials: list[Material] = []
 
         self.__sounds_file: SoundsFile = None
         self.__sound_definitions_file: SoundDefinitionsFile = None
@@ -1097,6 +1127,14 @@ class ResourcePack(Pack):
             
         return self.__animation_files
 
+    @cached_property
+    def material_files(self) -> list[MaterialFile]:
+        base_directory = os.path.join(self.input_path, "materials")
+        for local_path in glob.glob(base_directory + "/**/*.material", recursive=True):
+            local_path = os.path.relpath(local_path, self.input_path)
+            self.__material_files.append(MaterialFile(file_path = local_path, pack = self))
+        return self.__material_files
+
     @cached_property
     def entities(self) -> list[EntityFileRP]:
         base_directory = os.path.join(self.input_path, "entity")
@@ -1159,11 +1197,17 @@ class ResourcePack(Pack):
 
     @cached_property
     def models(self) -> list[Model]:
-        children = []
         for file in self.model_files:
             for child in file.models:
-                children.append(child)
-        return children
+                self.__models.append(child)
+        return self.__models
+
+    @cached_property
+    def materials(self) -> list[Material]:
+        for file in self.material_files:
+            for child in file.materials:
+                self.__materials.append(child)
+        return self.__materials
 
     @cached_property
     def sounds(self) -> list[str]:
@@ -1301,6 +1345,19 @@ class ResourcePack(Pack):
                 return acf
         raise AssetNotFoundError(f"AnimationControllerFileRP with path '{file_path}' does not exist.")
 
+    def get_material_file(self, file_path:str) -> MaterialFile:
+        for material_file in self.material_files:
+            if smart_compare(material_file.file_path, file_path):
+                return material_file
+        raise AssetNotFoundError(f"MaterialFile with path '{file_path}' does not exist.")
+
+    def get_material(self, id:str) -> Material:
+        for file_child in self.material_files:
+            for child in file_child.materials:
+                if smart_compare(child.id, id):
+                    return child
+        raise AssetNotFoundError(f"MaterialRP with id '{id}' does not exist.")
+
     def get_animation_file(self, file_path:str) -> AnimationFileRP:
         """
         Gets the AnimationFileRP with the given path.
@@ -1986,6 +2043,7 @@ class EntityFileRP(JsonFileResource):
         self.__animations: AnimationTriple = []
         self.__models: ModelTriple = []
         self.__textures: TextureDouble = []
+        self.__materials: MaterialTriple = []
 
     @property
     def identifier(self) -> str:
@@ -2044,6 +2102,37 @@ class EntityFileRP(JsonFileResource):
                 return child
         raise AssetNotFoundError(id)
 
+    @cached_property
+    def materials(self) -> list[MaterialTriple]:
+        for path, data in self.get_data_at("minecraft:client_entity/description/materials"):
+            self.__materials.append(MaterialTriple(parent = self, json_path = path, data = data))
+        return self.__materials
+    
+    def get_material(self, identifier:str) -> MaterialTriple:
+        """
+        Fetches a material resource, either by shortname, or material type.
+        """
+        for child in self.materials:
+            if smart_compare(child.shortname, identifier) or smart_compare(child.identifier, identifier):
+                return child
+        raise AssetNotFoundError(id)
+
+class MaterialFile(JsonFileResource):
+    """
+    MaterialFile is a class which represents a resource pack's material file.
+    Since many materials can be defined in the same file, it is often more useful
+    to use the MaterialRP class directly.
+    """
+    def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
+        super().__init__(data = data, file_path = file_path, pack = pack)
+        self.__materials: Material = []
+
+    @cached_property
+    def materials(self) -> list[Material]:
+        for path, data in self.get_data_at("**"):
+            self.__materials.append(Material(parent = self, json_path = path, data = data))
+        return self.__materials
+
 class AnimationFileRP(JsonFileResource):
     """
     AnimationFileRP is a class which represents a resource pack's animation file.
@@ -2156,6 +2245,9 @@ class RenderController(JsonSubResource):
     def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
         super().__init__(data=data, parent=parent, json_path=json_path)
 
+class Material(JsonSubResource):
+    def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
+        super().__init__(data=data, parent=parent, json_path=json_path)
 
 class Cube(JsonSubResource):
     def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
